// Class: AuthorString
// Author: Anton Gntsch
// Created 8-2001
// Modified 11-2001

// This class is intended to be used by a user interface (e.g. cold fusion template)
// to check the syntax of author team strings as defined for the Euro+Med project.
// The constructor class expects *one* author string (not author string concatenated
// with basionym authors). If the given authorstring is a basionym author string, no
// brackets should be included. Apart from the constructor AuthorString, a single
// public method CheckAuthorString is provided. This method returns a list of cardinal
// numbers corresponding to error message IDs in a database table (|-separated) as string.




import java.util.*;

public class AuthorString {
  private String Authors;

  private String  AuthorIdLst = "";

  public AuthorString(String AuthorString, String _AuthorIdLst) {
    Authors = AuthorString;
    AuthorIdLst = _AuthorIdLst;
  }

  public String CheckAuthorString() {
    String ListOfMessages = "";

    if (!CommonStuff.CommaSpaceOK(Authors)) {
      ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "4");
    }
    if (!CommonStuff.SpaceCommaOK(Authors)) {
      ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "5");
    }
    if (!CommonStuff.AmpersandSpaceOK(Authors)) {
      ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "6");
    }
    if (CommonStuff.ContainsStrangeChars(Authors, "Authors")) {
      ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "2");
    }
    if ((Authors.indexOf("et al") > -1) || (Authors.indexOf(" ex ") > -1) || (Authors.indexOf(" in ") > -1)) {
      if (Authors.indexOf("et al") > -1) {
        ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "3");
      }
      if (Authors.indexOf(" ex ") > -1) {
        ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "8");
      }
      if (Authors.indexOf(" in ") > -1) {
        ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "9");
      }
    }
    else {
      if (!CommaAmpersandOK()) {
        ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "37");
      }
      if (ExistsDotSpace()) {
        ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "7");
      }
     }
      // check if both enter unstructured and structured authorTeam.
      if ((Authors.length() != 0)&(AuthorIdLst.length()!= 0)){
         ListOfMessages = CommonStuff.AddMessage(ListOfMessages, "49");
    }
    return (ListOfMessages);
  }

  // Are commas and ampersands in the right order(..., ... & ...)?
  private boolean CommaAmpersandOK() {
    boolean Result;
    StringTokenizer AllTokens;
    String OneToken;
    int CountAmp;
    int CountComma;

    Result = true;
    CountAmp = 0;
    CountComma = 0;
    AllTokens = new StringTokenizer(Authors, ",&", true);
    while (AllTokens.hasMoreTokens()) {
      OneToken = AllTokens.nextToken();
      if (OneToken.equals(",") || OneToken.equals("&")) {
        if (OneToken.equals("&")) {
          CountAmp++;
        }
        if (OneToken.equals(",")) {
          if (CountAmp > 0) {
            Result = false;
          }
          CountComma++;
        }
      }
    }
    if ((CountComma > 0) && (CountAmp != 1)) {
      Result = false;
    }
    return(Result);
  }

  // Since initials of names must not be separated with spaces and commas must not be preceeded
  // by spaces, the combination ". " is not allowed within author strings.
  private boolean ExistsDotSpace() {
    boolean Result;
    int i;

    Result = false;
    for (i=0; i<=Authors.length()-1; i++) {
      if ((Authors.substring(i, i+1) == ".") && (Authors.substring(i+1, i+2) == " ")) {
        Result = true;
      }
    }
    return(Result);
  }


}