// Class: BotName
// Author: Anton Gntsch, J. Li
// Created 8-2001
// Modified 8-2002

// This class is intended to be called by a user interface (e.g. ColdFusion form to
// check the fields of a plantname as entered by a user for correctness. The fields
// correspond to the datastructure of the editor backbone database (similar to TDWG
// structure for botanical names. The constructor is called with a string containing the
// list of fields as fieldname=content pairs separatet with the pipe symbol (|). See below
// for a list of valid fieldnames.
// Currently, no exception handling is implemented. Therefore, the constructor should be called
// with a well formed argument string. If a form field does not contain data the field must not
// be transmitted. Therefore, do not include Genus= or Genus="" for example. An example for a
// well formed argument string is:
//
// "Rank=60|Genus=Abies|SpEpi=alba x abla|BinomHybFlag=x"
//
// Appart from the constructor BotName(String), only the CheckAll() method is public. CheckAll
// returns a |-separated list of cardinal numbers corresponding to error message IDs in a data-
// base table. Other methods can be switched from private to public if needed.

import java.util.*;

public class BotName {
  private int Rank;
  private String SupraGenericName = "";
  private String UnnamedNamePhrase = "";
  private String Genus = "";
  private String SpeciesEpi = "";
  private String GenusSubdivisionEpi = "";
  private String InfraSpeciesEpi = "";
  private String CultivarName = "";
  private String CultivarGroupName = "";
  private String BasAuthor = "";
  private String Author = "";
  private String MonomHybFlag = "";
  private String BinomHybFlag = "";
  private String TrinomHybFlag = "";


  // constructor: Parse given String
  public BotName(String AllArguments) {
    StringTokenizer StAllArguments = new StringTokenizer(AllArguments, "|=");
    String OneToken;
    int EqIndex;
    String ArgName;
    String ArgValue;
    String MoveNext;

    while (StAllArguments.hasMoreTokens()) {
          ArgName = StAllArguments.nextToken();
          if (ArgName.equals("Rank")) {
            Rank = Integer.parseInt(StAllArguments.nextToken());
          }
          if (ArgName.equals("UnnamedNamePhrase")) {
            UnnamedNamePhrase = StAllArguments.nextToken();
          }
          if (ArgName.equals("SupraGenericName")) {
            SupraGenericName = StAllArguments.nextToken();
          }
          if (ArgName.equals("Genus")) {
            Genus = StAllArguments.nextToken();
          }
           if (ArgName.equals("GenusSubdivisionEpi")) {
            GenusSubdivisionEpi = StAllArguments.nextToken();
          }
          if (ArgName.equals("SpeciesEpi")) {
       SpeciesEpi = StAllArguments.nextToken();
          }
          if (ArgName.equals ("InfraSpeciesEpi")) {
            InfraSpeciesEpi = StAllArguments.nextToken();
          }

          if (ArgName.equals("CultivarName")) {
            CultivarName = StAllArguments.nextToken();
          }
          if (ArgName.equals("CultivarGroupName")) {
            CultivarGroupName = StAllArguments.nextToken();
          }
           if (ArgName.equals("BasAuthor")) {
            BasAuthor = StAllArguments.nextToken();
          }
           if (ArgName.equals("Author")) {
            Author = StAllArguments.nextToken();
          }
           if (ArgName.equals("BinomHybFlag")) {
            BinomHybFlag = StAllArguments.nextToken();
          }
          if (ArgName.equals("MonomHybFlag")) {
            MonomHybFlag = StAllArguments.nextToken();
          }
           if (ArgName.equals("TrinomHybFlag")) {
            TrinomHybFlag = StAllArguments.nextToken();
          }

    }
  }

  public String CheckAll() {
    String Messages = "";

    // supra generic name
    if (Rank < 40) {
      Messages = CommonStuff.AddMessage(Messages, CheckSupraGenericName());
    }
    // Genus
    if (Rank == 40) {
      Messages = CommonStuff.AddMessage(Messages, CheckGenus());
      Messages = CommonStuff.AddMessage(Messages, CheckCultivar());
    }
    // ranks between Genus and aggregates
    if ((Rank > 40) && (Rank < 58)) {
      Messages = CommonStuff.AddMessage(Messages, CheckGenus());
      Messages = CommonStuff.AddMessage(Messages, CheckGenusSubdivisionEpi());
    }
    // species, aggregates and group
    if ((Rank == 58) || (Rank == 59)) {
      Messages = CommonStuff.AddMessage(Messages, CheckGenus());
     Messages = CommonStuff.AddMessage(Messages, CheckSpeciesEpi());
    }

     if (Rank == 60) {
      Messages = CommonStuff.AddMessage(Messages, CheckGenus());
     Messages = CommonStuff.AddMessage(Messages, CheckSpeciesEpi());
     Messages = CommonStuff.AddMessage(Messages, CheckCultivar());

    }
    // ranks below species
    if (Rank > 60) {
      Messages = CommonStuff.AddMessage(Messages, CheckGenus());
     Messages = CommonStuff.AddMessage(Messages, CheckSpeciesEpi());
      Messages = CommonStuff.AddMessage(Messages, CheckInfraSpeciesEpi());
    }
    return (Messages);
  }

  // Some methods to check the name field's syntax

  private String CheckSupraGenericName() {
    String Messages = "";

    if (SupraGenericName == "") {
      Messages = CommonStuff.AddMessage(Messages, "36");
    } else {
      if (SupraGenericName.length() < 2) {
        Messages = CommonStuff.AddMessage(Messages, "27");
      }
      if (CommonStuff.ContainsStrangeChars(SupraGenericName, "LatinName")) {
       Messages = CommonStuff.AddMessage(Messages, "10");
      }
      if (!CommonStuff.HasFirstUpperCase(SupraGenericName)) {
        Messages = CommonStuff.AddMessage(Messages, "11");
      }
      if (CommonStuff.ContainsSpaces(SupraGenericName)) {
        Messages = CommonStuff.AddMessage(Messages, "12");
      }
    }
    return(Messages);
  }

  private String CheckGenus() {
    String Messages = "";

    if (Genus == "") {
      Messages = CommonStuff.AddMessage(Messages, "32");
    } else {
      if (Genus.length() < 2) {
        Messages = CommonStuff.AddMessage(Messages, "28");
      }
      if (CommonStuff.ContainsStrangeChars(Genus, "LatinName")) {
        Messages = CommonStuff.AddMessage(Messages, "13");
      }
      if (!MonomHybFlag.equals("x")) {
        if (!CommonStuff.HasFirstUpperCase(Genus)) {
          Messages = CommonStuff.AddMessage(Messages, "14");
        }
        if (CommonStuff.ContainsSpaces(Genus)) {
          Messages = CommonStuff.AddMessage(Messages, "15");
        }
      } else {
        if (!ValidGenusHybridStructure()){
          Messages = CommonStuff.AddMessage(Messages, "16");
        }
      }
    }
    return(Messages);
  }

  private String CheckSpeciesEpi() {
    String Messages = "";

    if (SpeciesEpi == "") {
      Messages = CommonStuff.AddMessage(Messages, "33");
    } else {
      if (SpeciesEpi.length() < 2) {
        Messages = CommonStuff.AddMessage(Messages, "29");
      }
      if (CommonStuff.ContainsStrangeChars(SpeciesEpi, "LatinName")) {
        Messages = CommonStuff.AddMessage(Messages, "17");
      }
      if (!BinomHybFlag.equals("x")) {
        if (!CommonStuff.IsAllLowerCase(SpeciesEpi)) {
          Messages = CommonStuff.AddMessage(Messages, "18");
        }
        if (CommonStuff.ContainsSpaces(SpeciesEpi)) {
          Messages = CommonStuff.AddMessage(Messages, "19");
        }
      } else {
        if (!ValidSpeciesHybridStructure()){
          Messages = CommonStuff.AddMessage(Messages, "20");
        }
      }
    }
    return(Messages);
  }

  private String CheckInfraSpeciesEpi() {
    String Messages = "";

    if (InfraSpeciesEpi == "") {
      Messages = CommonStuff.AddMessage(Messages, "34");
    } else {
      if (InfraSpeciesEpi.length() < 2) {
        Messages = CommonStuff.AddMessage(Messages, "30");
      }
      if (CommonStuff.ContainsStrangeChars(InfraSpeciesEpi, "LatinName")) {
        Messages = CommonStuff.AddMessage(Messages, "21");
      }
      if (CommonStuff.ContainsSpaces(InfraSpeciesEpi)) {
          Messages = CommonStuff.AddMessage(Messages, "22");
      }
      if (!CommonStuff.IsAllLowerCase(InfraSpeciesEpi)) {
          Messages = CommonStuff.AddMessage(Messages, "23");
      }
    }
    return(Messages);
  }

  private String CheckGenusSubdivisionEpi() {
    String Messages = "";

    if (GenusSubdivisionEpi == "") {
      Messages = CommonStuff.AddMessage(Messages, "35");
    } else {
      if (GenusSubdivisionEpi.length() < 2) {
        Messages = CommonStuff.AddMessage(Messages, "31");
      }
      if (CommonStuff.ContainsStrangeChars(GenusSubdivisionEpi, "LatinName")) {
        Messages = CommonStuff.AddMessage(Messages, "24");
      }
      if (CommonStuff.ContainsSpaces(GenusSubdivisionEpi)) {
          Messages = CommonStuff.AddMessage(Messages, "25");
      }
      if (!CommonStuff.HasFirstUpperCase(GenusSubdivisionEpi)) {
          Messages = CommonStuff.AddMessage(Messages, "26");
      }
    }
    return(Messages);
  }

   private String CheckCultivar() {
    String Messages = "";

   // check if both enter cultivar part  and  authorTeam.
   if (((Author.length()!= 0)||(BasAuthor.length()!= 0))&((CultivarName.length()!= 0)||(CultivarGroupName.length()!= 0))){
         Messages = CommonStuff.AddMessage(Messages, "51");
    }

    if ((UnnamedNamePhrase.length() != 0)&((CultivarName.length()!= 0)||(CultivarGroupName.length()!= 0))){
         Messages = CommonStuff.AddMessage(Messages, "50");
    }
     return(Messages);
  }


  // Check whether the given string has the form "aaa x bbb" or "x aaa".
  private boolean ValidSpeciesHybridStructure() {
    boolean Result;
    StringTokenizer NameSubstr = new StringTokenizer(SpeciesEpi, " ");

    Result = true;
    if (NameSubstr.countTokens() == 1) {
      if (!CommonStuff.IsAllLowerCase(SpeciesEpi)) {
          Result = false;
      }
    } else if (NameSubstr.countTokens() == 3) {
      if (!(CommonStuff.IsAllLowerCase(NameSubstr.nextToken()))) {
        Result = false;
      }
      if (!NameSubstr.nextToken().equals("x")) {
        Result = false;
      }
      if (!(CommonStuff.IsAllLowerCase(NameSubstr.nextToken()))) {
        Result = false;
      }
    } else {
      Result = false;
    }
    return(Result);
  }

  // Check whether the given string has the form "Aaa x Bbb" or "x Aaa".
  private boolean ValidGenusHybridStructure() {
    boolean Result;
    StringTokenizer NameSubstr = new StringTokenizer(Genus, " ");

    Result = true;
    if (NameSubstr.countTokens() == 1) {
      if (!(CommonStuff.HasFirstUpperCase(NameSubstr.nextToken()))) {
        Result = false;
      }
    } else if (NameSubstr.countTokens() == 3) {
      if (!(CommonStuff.HasFirstUpperCase(NameSubstr.nextToken()))) {
        Result = false;
      }
      if (!NameSubstr.nextToken().equals("x")) {
        Result = false;
      }
      if (!(CommonStuff.HasFirstUpperCase(NameSubstr.nextToken()))) {
        Result = false;
      }
    } else {
      Result = false;
    }
    return(Result);
  }

  // Check whether the given string has the form "x aaa" or "X aaa"
  private boolean HasNamedHybridMarker(String Arg) {
    if ((Arg.indexOf("x ") > -1) || (Arg.indexOf("X ") > -1)) {
      return(true);
    } else {
      return(false);
    }
  }

} // Class BotName